#!/usr/bin/env python

import matplotlib.pyplot as plt
import csv
import numpy as np

# loadxy loads (x int; y float) vectors from a csv file
def loadxy(path):
    x = []
    y = []
    with open(path, 'r') as f:
        csvf = csv.reader(f, delimiter='\t')
        for row in csvf:
            x.append(int(row[0]))
            y.append(float(row[1]))

    return x, y

def plotxy(xy, **kw):
    x, y = xy
    plt.plot(x, y, **kw)

pread_c  = loadxy("t_syspread_c.csv")
pread_go = loadxy("t_syspread_go.csv")
copy_c   = loadxy("t_copy_c.csv")
copy_go  = loadxy("t_copy_go.csv")


plt.title("Timing for syspread & memcopy")
plotxy(pread_c,  label="syspread/c")
plotxy(pread_go, label="syspread/go")
plotxy(copy_c,   label="copy/c")
plotxy(copy_go,  label="copy/go")

assert pread_c[0] == copy_c[0]
plotxy((pread_c[0], np.array(pread_c[1]) - np.array(copy_c[1])), label="syspread/c - copy/c")

plt.legend()
plt.ylabel('time (ns)')
plt.xlabel('buffer size (bytes)')
#plt.show()
plt.savefig('syspread.png')
