#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/user.h>
#include <sys/mman.h>

//		 12345678
#define	NITER	100000

// microtime returns current time as double
double microtime() {
	int err;
	struct timeval tv;

	err = gettimeofday(&tv, NULL);
	if (err == -1) {
		perror("gettimeofday");
		abort();
	}

	return tv.tv_sec + 1E-6 * tv.tv_usec;
}


int main() {
	unsigned char *addr, sum = 0;
	int fd, err, i;
	size_t size;
	double Tstart, Tend;

	fd = open("/dev/shm/y.dat", O_RDWR);
	if (fd == -1) {
		perror("open");
		abort();
	}

	size = NITER * PAGE_SIZE;

	err = ftruncate(fd, size);
	if (err == -1) {
		perror("ftruncate");
		abort();
	}

	// make sure RAM is actually allocated
	Tstart = microtime();
	err = fallocate(fd, /*mode*/0, 0, size);
	Tend = microtime();
	if (err == -1) {
		perror("fallocate");
		abort();
	}
	printf("T(fallocate): %.1f\n", Tend - Tstart);	// XXX print per page?

	addr = mmap(NULL, size, PROT_READ, MAP_SHARED, fd, 0);
	if (addr == MAP_FAILED) {
		perror("mmap");
		abort();
	}

	Tstart = microtime();
	for (i=0; i<NITER; i++) {
		sum += addr[i*PAGE_SIZE];
	}
	Tend = microtime();

	printf("T(pagefault): %.1f\t(%i)\n", (Tend - Tstart) / NITER, sum);

	return 0;
}
